@@ -51,21 +51,31 @@ def cache_run(con, cur, input_word, req_url, dict):
     # data is a tuple (response_url, response_text) if any
     data = get_cache(con, cur, input_word, req_url)
 
-    if data is not None:
-        res_url, res_text = data
-
-        if DICTS[0].lower() in res_url:
-            print(f'{OP[5]} "{input_word}" from {DICTS[0]} in cache. Not to use it, try again with -f or --fresh option')
-            logger.debug(f"{OP[1]} {res_url}")
-            soup = make_a_soup(res_text)
-            cambridge.parse_and_print(soup, res_url)
+    if data is None:
+        if "s" != input_word[-1]:
+            return False
         else:
-            print(f'{OP[5]} "{input_word}" from {DICTS[1]} in cache. Not to use it, try again with -f or --fresh option')
-            nodes = webster.parse_dict(res_text, True, res_url, False)
-            webster.parse_and_print(nodes, res_url)
-        return True
+            data = get_cache(con, cur, input_word[:-1], req_url)
+            if data is None:
+                if "es" != input_word[-2:]:
+                    return False
+                else:
+                    data = get_cache(con, cur, input_word[:-2], req_url)
+                    if data is None:
+                        return False
 
-    return False
+    res_url, res_word, res_text = data
+
+    if DICTS[0].lower() in res_url:
+        print(f'{OP[5]} "{res_word}" from {DICTS[0]} in cache. Not to use it, try again with -f or --fresh option')
+        logger.debug(f"{OP[1]} {res_url}")
+        soup = make_a_soup(res_text)
+        cambridge.parse_and_print(soup, res_url)
+    else:
+        print(f'{OP[5]} "{res_word}" from {DICTS[1]} in cache. Not to use it, try again with -f or --fresh option')
+        nodes = webster.parse_dict(res_text, True, res_url, False)
+        webster.parse_and_print(nodes, res_url)
+    return True
 
 
 def save(con, cur, input_word, response_word, response_url, response_text):
@@ -87,20 +97,32 @@ def save(con, cur, input_word, response_word, response_url, response_text):
 def print_spellcheck(con, cur, input_word, suggestions, dict, is_ch=False):
     """Parse and print spellcheck info."""
 
-    console.print("[red]" + "\n!!! " + "[/red]" + "[yellow]" + input_word.upper() + "[/yellow]" + " you've entered isn't in the dictionary.\n")
+    if dict == DICTS[1]:
+        console.print("[red bold]" + input_word.upper() + "[/red bold]" + " you've entered isn't in the " + "[blue]" + dict + "[/blue]" + " dictionary.\n")
+    else:
+        console.print("[red bold]" + input_word.upper() + "[/red bold]" + " you've entered isn't in the " + "[yellow]" + dict + "[/yellow]" + " dictionary.\n")
 
     for count, sug in enumerate(suggestions):
-        console.print("[bold]%2d" % (count+1), end="")
-        console.print("[#3C8DAD] %s" % sug)
+        console.print("[bold][%2d]" % (count+1), end="")
+        if dict == DICTS[1]:
+            console.print("[blue] %s" % sug)
+        else:
+            console.print("[yellow] %s" % sug)
    
+    console.print("\nEnter the [bold]NUMBER[/bold] above to [u]look up[/u] the word suggestion || [bold]LOWER-CASE LETTER T[/bold] to [u]toggle[/u] dictionary || [bold]ANY OTHER KEY[/bold] to [u]exit[/u]:")
 
-    console.print("[green]" + "\n>>> " + "[/green]" + "Enter the NUMBER above to look up the word suggestion OR any other KEY to exit:")
-    key = input("")
+    key = input("You typed: ")
+    print()
 
     if key.isnumeric() and (1 <= int(key) <= len(suggestions)):
         if dict == DICTS[1]:
             webster.search_webster(con, cur, suggestions[int(key) - 1])        
         if dict == DICTS[0]:
             cambridge.search_cambridge(con, cur, suggestions[int(key) - 1], False, is_ch)
+    elif key == "t":
+        if dict == DICTS[0]:
+            webster.search_webster(con, cur, input_word)        
+        if dict == DICTS[1]:
+            cambridge.search_cambridge(con, cur, input_word, False, is_ch)
     else:
         sys.exit()
\ No newline at end of file
