@@ -78,12 +78,6 @@ def fresh_run(con, cur, req_url, input_word):
     nodes = parse_dict(res_text, found, res_url, True)
 
     if found:
-        # res_word may not be returned when `save` is called by concurrency
-        # word wiin res_url is still the input_word formatted
-        global res_word
-        if not res_word:
-            res_word = input_word
-
         parse_thread = threading.Thread(
             target=parse_and_print, args=(nodes, res_url,)
         )
@@ -137,6 +131,14 @@ def parse_dict(res_text, found, res_url, is_fresh):
             logger.error("The fetched content is not intended for the word, due to your network or the website reasons, please try again.")
             sys.exit()
 
+
+        result = tree.xpath("//*[@id='dictionary-entry-1']/div[1]/div/div[1]/h1/text()")
+        if not result:
+            result = tree.xpath("//*[@id='dictionary-entry-1']/div[1]/div/div/h1/span/text()")
+
+        global res_word
+        res_word = result[0]
+
         ## NOTE: [only for debug]
         # for node in nodes:
         #     try:
@@ -174,7 +176,8 @@ def nearby_entries(node):
                 console.print(f"[{webster_color.bold} {webster_color.nearby_title}]{elm.text}", end="")
 
             if has_em:
-                console.print(f"[{webster_color.bold} {webster_color.italic} {webster_color.nearby_em}]{elm.text}", end="\n")
+                word = "".join(list(elm.itertext()))
+                console.print(f"[{webster_color.bold} {webster_color.italic} {webster_color.nearby_em}]{word}", end="\n")
 
             if has_word:
                 console.print(f"[{webster_color.nearby_word}]{elm.text}", end="\n")
@@ -211,11 +214,11 @@ def synonyms(node):
                 total_num = len(children)
                 
                 for index, child in enumerate(children):
-                    text = "".join(list(child.itertext())).strip()
+                    syn = "".join(list(child.itertext())).strip()
                     if index != (total_num - 1):
-                        console.print(f"[{webster_color.syn_item}]{text},", end=" ")
+                        console.print(f"[{webster_color.syn_item}]{syn},", end=" ")
                     else:
-                        console.print(f"[{webster_color.syn_item}]{text}", end=" ")
+                        console.print(f"[{webster_color.syn_item}]{syn}", end=" ")
 
 
 ###########################################
@@ -267,11 +270,22 @@ def phrases(node):
     children = node.getchildren()[1]
     
     for child in children:
-        if child.attrib["class"] == "drp":
-            console.print(f"[{webster_color.ph_item} {webster_color.bold}]{child.text}")
+        try:
+            if child.attrib["class"] == "drp":
+                if child.getnext().tag == "span":
+                    console.print(f"[{webster_color.ph_item} {webster_color.bold}]{child.text}", end = "")
+                else:
+                    console.print(f"[{webster_color.ph_item} {webster_color.bold}]{child.text}", end = "\n")
 
-        if child.attrib["class"] == "vg":
-            vg(child)
+            if child.attrib["class"] == "vg":
+                vg(child)
+
+        except KeyError:
+            for i in child.getchildren():
+                if i.attrib["class"] == "vl":
+                    print_or_badge(i.text)
+                else:
+                    console.print(f"[{webster_color.ph_item} {webster_color.bold}]{i.text}", end = "\n")
 
 
 ###########################################
@@ -327,12 +341,7 @@ def dtText(node, ancestor_attr, count):
     texts = list(node.itertext())
 
     if count != 1:
-        if ancestor_attr == "sense has-sn has-num-only":
-            console.print(f"\n  ", end="")
-        if ancestor_attr == "sense has-sn has-num" or ancestor_attr == "sense has-sn":
-            console.print(f"\n    ", end="")
-        if ancestor_attr == "sense  no-subnum":
-            console.print(f"\n", end="")
+        format_basedon_ancestor(ancestor_attr, prefix="\n")
 
     for text in texts:
         if text == " ":
@@ -341,19 +350,18 @@ def dtText(node, ancestor_attr, count):
         if text == ": ":
             text = text.strip()
 
-        if "sense" in text:
-            text = " " + text
+        # FIXME when meaning similar to another word's sense 1, 2, ... "another word" and "sense" squeezes together
+        # if "sense" in text:
+        #     text = " " + text
+        print_meaning_content(text, end="")
 
-        console.print(f"[{webster_color.meaning_content}]{text}", end = "")
+    console.print("", end = " ")
 
 
 def ex_sent(node, ancestor_attr):
-    if ancestor_attr == "sense has-sn has-num-only":
-        console.print(f"\n  [{webster_color.accessory} {webster_color.bold}]|", end="")
-    if ancestor_attr == "sense has-sn has-num" or ancestor_attr == "sense has-sn":
-        console.print(f"\n    [{webster_color.accessory} {webster_color.bold}]|", end="")
-    if ancestor_attr == "sense  no-subnum":
-        console.print(f"\n[{webster_color.accessory} {webster_color.bold}]|", end="")
+    format_basedon_ancestor(ancestor_attr, prefix="\n")
+
+    console.print(f"[{webster_color.accessory} {webster_color.bold}]|", end="")
 
     for text in node.itertext():
         if text.strip():
@@ -364,6 +372,7 @@ def sub_content_thread(node, ancestor_attr):
     children = node.getchildren()
     for child in children:
         attr = child.attrib["class"]
+
         if ("ex-sent" in attr) and ("aq has-aq" not in attr):
             ex_sent(child, ancestor_attr)
 
@@ -371,7 +380,7 @@ def sub_content_thread(node, ancestor_attr):
             elm = child.getchildren()[0].getchildren()[0]
             elm_attr = elm.attrib["class"]
             if ("ex-sent" in elm_attr) and ("aq has-aq" not in elm_attr):
-                ex_sent(child, ancestor_attr)
+                ex_sent(elm, ancestor_attr)
             
 
 def dt(node, ancestor_attr, self_attr):
@@ -383,24 +392,31 @@ def dt(node, ancestor_attr, self_attr):
         
         if child_attr == "sd": # label before meaning content
             if self_attr == "sdsense":
-                if "has-num-only" in ancestor_attr: 
-                    console.print(f"  [{webster_color.italic} {webster_color.meaning_badge}]{child.text}", end=" ")
-                elif "no-subnum" in ancestor_attr:
-                    console.print(f"[{webster_color.italic} {webster_color.meaning_badge}]{child.text}", end=" ")
-                else:
-                     console.print(f"    [{webster_color.italic} {webster_color.meaning_badge}]{child.text}", end=" ")
+                format_basedon_ancestor(ancestor_attr, prefix="")
+                print_meaning_badge(child.text)
+
+
             else:
-                console.print(f"[{webster_color.italic} {webster_color.meaning_badged}]{child.text}", end=" ")
+                print_meaning_badge(child.text)
         
         if child_attr == "dtText":
             dtText(child, ancestor_attr, dtText_count)   # only meaning text
             dtText_count += 1
 
         if child_attr == "uns":
+            if child.getprevious().attrib["class"] == "sub-content-thread":
+                print()
+
             elms = child.getchildren()[0].getchildren()
             for elm in elms:
                 if elm.attrib["class"] == "unText":
-                    console.print(f" [{webster_color.italic} {webster_color.meaning_badge}]{elm.text}", end=" ")
+                    text = "".join(list(elm.itertext())).strip()
+                    if "mdash" in elm.getprevious().attrib["class"]:
+                        print_meaning_badge(text)
+                    else:
+                        format_basedon_ancestor(ancestor_attr, prefix="")
+                        print_meaning_badge(text)
+
                 if elm.attrib["class"] == "sub-content-thread":
                     sub_content_thread(elm, ancestor_attr)
             
@@ -409,20 +425,37 @@ def dt(node, ancestor_attr, self_attr):
     print()
             
 
-def sense(node):
-    attr = node.attrib["class"]
+def sense(node, attr, parent_attr, ancestor_attr):
     children = node.getchildren()
 
+    # meaning without any sign
     if attr == "sense  no-subnum":
         sense_content = children[0] # class "sense-content w-100"
 
-    # meaning with "a", "b" letter,  with or without "1", "2" (having siblings before it)
-    if attr == "sense has-sn has-num" or attr == "sense has-sn":
+    # meaning with "1" + "a"
+    if attr == "sense has-sn has-num":
         sn = children[0].getchildren()[0].text
-        if "has-num" in attr:
+        if "has-subnum" in ancestor_attr and "sb-0" not in parent_attr:
+            console.print(f"  [{webster_color.bold} {webster_color.meaning_letter}]{sn}", end = " ")
+        else:
+            console.print(f"[{webster_color.bold} {webster_color.meaning_letter}]{sn}", end = " ")
+
+        sense_content = children[1]  # class "sense-content w-100"
+
+    # meaing with only "b" or "1" + "a" + "(1)", or "1" + "a"
+    if attr == "sense has-sn": 
+        sn = children[0].getchildren()[0].text
+
+        if ("has-subnum" in ancestor_attr and "sb-0" in parent_attr) or ("no-sn letter-only" in ancestor_attr):
             console.print(f"[{webster_color.bold} {webster_color.meaning_letter}]{sn}", end = " ")
         else:
             console.print(f"  [{webster_color.bold} {webster_color.meaning_letter}]{sn}", end = " ")
+
+        sense_content = children[1]  # class "sense-content w-100"
+
+    # meaning with only (2)   
+    if attr == "sense has-num-only has-subnum-only":
+        console.print(f"    ", end = "")
         sense_content = children[1]  # class "sense-content w-100"
 
     # meaning with only number
@@ -430,20 +463,38 @@ def sense(node):
         sense_content = children[1]  # class "sense-content w-100"
 
     elms = sense_content.getchildren()
-    
     for elm in elms:
         elm_attr = elm.attrib["class"]
         if "badge" in elm_attr:
             text = "".join(list(elm.itertext())).strip()
-            console.print(f"[{webster_color.italic} {webster_color.meaning_badge}]{text}", end=" ")
+            print_meaning_badge(text)
 
         if elm_attr == "dt " or elm_attr == "dt hasSdSense" or elm_attr == "sdsense":
             dt(elm, attr, elm_attr)
-            
 
-def sb_entry(node):
+        if elm_attr == "et":
+            texts = list(elm.itertext())
+            for index, text in enumerate(texts):
+                if index == 0:
+                    text = text.replace("\n", "")
+                    print_meaning_content(text, end="")
+                elif index == len(texts) - 1:
+                    print_meaning_content(text, end=" ")
+                else:
+                    print_meaning_content(text, end="")
+
+
+def sb_entry(node, parent_attr):
     child = node.getchildren()[0]
-    sense(child)
+    attr = node.attrib["class"]         # "sb-0 sb-entry"
+    child_attr = child.attrib["class"]  # "sense has-sn" or "pseq no-subnum"
+    if "pseq" in child_attr:
+        elms = child.getchildren()[0].getchildren()
+        for e in elms:
+            e_attr = e.attrib["class"]  # "sense has-sn"
+            sense(e, e_attr, attr, parent_attr)        # sense(child, "sense has-sn", "sb-0 sb-entry", "....")
+    else:
+        sense(child, child_attr, attr, parent_attr)    # sense(child, "sense has-sn", "sb-0 sb-entry, "sb has-num has-let ms-lg-4 ms-3 w-100")
 
 
 def vg_sseq_entry_item(node):
@@ -451,15 +502,21 @@ def vg_sseq_entry_item(node):
 
     children = node.getchildren()
     for child in children:
+        attr = child.attrib["class"]
         # print number label if any
-        if child.attrib["class"] == "vg-sseq-entry-item-label":
+        if attr == "vg-sseq-entry-item-label":
             console.print(f"[{webster_color.bold} {webster_color.meaning_num}]{child.text}", end=" ")
 
         # print meaning content
-        if "ms-lg-4 ms-3 w-100" in child.attrib["class"]:
-            for i in child:
+        if "ms-lg-4 ms-3 w-100" in attr:
+            for c in child.iterchildren():
+                if c.getchildren()[0].attrib["class"] == "sen has-num-only":
+                    print_meaning_badge(c.getchildren()[0][1].text)
+                    print()
+                    continue
+                    
                 # print class "sb-0 sb-entry", "sb-1 sb-entry" ...
-                sb_entry(i) 
+                sb_entry(c, attr) 
 
 
 def vg(node):
@@ -474,7 +531,7 @@ def vg(node):
         # print transitive or intransitive
         if child.attrib["class"] == "vd firstVd" or child.attrib["class"] == "vd":
             e = child.getchildren()[0]
-            console.print(f"\n[{webster_color.tran}]{e.text}")
+            console.print(f"[{webster_color.tran}]{e.text}")
 
 
 # --- parse class "row entry-header" --- #
@@ -483,7 +540,7 @@ def entry_header_content(node):
 
     for elm in node.iterchildren():
         if elm.tag == "h1" or elm.tag == "p":
-            word = elm.text
+            word = "".join(list(elm.itertext()))
             global word_entries
             word_entries.append(word)
             console.print(f"[{webster_color.eh_h1_word} {webster_color.bold}]{word}", end=" ")
@@ -508,8 +565,7 @@ def entry_attr(node):
                     console.print(f"[{webster_color.eh_word_syllables}]{syllables}", end=" ")
 
                 if i.tag == "span" and "prons-entries-list-inline" in i.attrib["class"]:
-                    pron = list(i.itertext())[1].strip()
-                    console.print(f"[{webster_color.eh_pron}]|{pron}|", end="\n")
+                    print_pron(i, end="\n")
 
 
 def row_entry_header(node):
@@ -540,10 +596,15 @@ def entry_uros(node):
 def row_headword_row_header_ins(node):
     """Print verb types. e.g. valued; valuing"""
 
-    children = node.getchildren()[0][0]
+    children = node.getchildren()[0].getchildren()[0]
     for child in children:
-        if child.attrib["class"] == "il  il-badge badge mw-badge-gray-100":
+        attr = child.attrib["class"]
+        if attr == "il  il-badge badge mw-badge-gray-100":
             console.print(f"[{webster_color.bold} {webster_color.italic} {webster_color.wt}]{child.text.strip()}", end=" ")
+        elif attr == "prt-a":
+            print_pron(child, end="")
+        elif attr == "il ":
+            print_or_badge(child.text)
         else:
             console.print(f"[{webster_color.wt}]{child.text}", end="")
 
@@ -574,7 +635,7 @@ def dictionary_entry(node):
     for elm in node.iterchildren():
         try: 
             if elm.attrib["class"]:
-                if elm.attrib["class"] == "row entry-header":
+                if "row entry-header" in elm.attrib["class"]:
                     row_entry_header(elm)
 
                 if elm.attrib["class"] == "row headword-row header-ins":
@@ -588,10 +649,47 @@ def dictionary_entry(node):
 
                 if elm.attrib["class"] == "dxnls":
                     see_also(elm)
+                
+                if elm.attrib["class"] == "mt-3":
+                    badge = elm.getchildren()[0]  # class with "badge mw-badge"
+                    print_meaning_badge(badge.text)
+                    print()
+
         except:
             continue
 
 
+##############################
+# --- print abstractions --- #
+##############################
+
+def print_meaning_badge(text):
+    console.print(f"[{webster_color.italic} {webster_color.meaning_badge}]{text}", end=" ")
+
+
+def print_meaning_content(text, end=""):
+    console.print(f"[{webster_color.meaning_content}]{text}", end = end)
+
+
+def format_basedon_ancestor(ancestor_attr, prefix="", suffix=""):
+    console.print(prefix, end="")
+    if ancestor_attr == "sense has-sn has-num-only":
+        console.print(f"  ", end=suffix)
+    if ancestor_attr == "sense has-sn has-num" or ancestor_attr == "sense has-sn":
+        console.print(f"    ", end=suffix)
+    if ancestor_attr == "sense  no-subnum":
+        console.print(f"", end=suffix)
+    if ancestor_attr == "sense has-num-only has-subnum-only":
+        console.print(f"    ", end=suffix)
+
+def print_pron(node, end=""):
+    pron = list(node.itertext())[1].strip()
+    console.print(f"[{webster_color.eh_pron}]|{pron}|", end=end)
+
+def print_or_badge(text):
+    console.print(f"[{webster_color.badge}]{text}", end = "")
+
+
 #####################################################
 # --- Entry point of all prints of a word found --- #
 #####################################################
@@ -629,8 +727,4 @@ def parse_and_print(nodes, res_url):
             related_phrases(node, word_entries)
 
     dict_name = "The Merriam-Webster Dictionary"
-    console.print(f"\n[{webster_color.dict_name}]{dict_name}", justify="right")
-
-    global res_word
-    if word_entries:
-        res_word = word_entries[0]
+    console.print(f"[{webster_color.dict_name}]{dict_name}", justify="right")
